"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectXPathInjection = detectXPathInjection;
const XPATH_INJECTION_PATTERNS = [
    /' or '1'='1/i,
    /" or "1"="1/i,
    /' or 'a'='a/i,
    /" or "a"="a/i,
    /' or '(.+?)'='(.+?)'/i,
    /" or "(.+?)"="(.+?)"/i,
    /\bconcat\((.*?)\)/i,
    /' or count\((.*?)\) > 0/i,
    /" or count\((.*?)\) > 0/i,
    /' or contains\((.*?)\)/i,
    /" or contains\((.*?)\)/i,
    /' or starts-with\((.*?)\)/i,
    /" or starts-with\((.*?)\)/i,
    /' or ends-with\((.*?)\)/i,
    /" or ends-with\((.*?)\)/i,
    /\bsubstring\((.*?)\)/i,
    /\/\*/i,
    /\.\.\/\.\./i,
    /\/\.\.\//i,
    /\bor\b|\band\b/i,
    /\btext\(\)/i,
    /\[.*?\]/i,
    /not\((.*?)\)/i,
    /\$\{.*?\}/i,
    /\bnamespace\b/i,
];
/**
 * Detects if there is a potential XPath injection in a given input.
 *
 * @param input - The input to analyze.
 * @returns `true` if a potential XPath injection is detected, otherwise `false`.
 */
function detectXPathInjection(input) {
    if (typeof input === 'string') {
        return XPATH_INJECTION_PATTERNS.some((pattern) => pattern.test(input));
    }
    else if (typeof input === 'object' && input !== null) {
        return checkObjectForXPathInjection(input).length > 0;
    }
    else if (Array.isArray(input)) {
        return input.some((item) => detectXPathInjection(item));
    }
    return false;
}
/**
 * Recursively checks an object or array for potential XPath injections.
 *
 * @param obj - The object or array to analyze.
 * @param path - The current path within the object (for reporting purposes).
 * @returns An array of strings describing potential security issues found.
 */
function checkObjectForXPathInjection(obj, path = '') {
    let issues = [];
    if (typeof obj === 'string') {
        if (XPATH_INJECTION_PATTERNS.some((pattern) => pattern.test(obj))) {
            issues.push(`XPath Injection detected at ${path}: "${obj}"`);
        }
    }
    else if (Array.isArray(obj)) {
        obj.forEach((item, index) => {
            issues = issues.concat(checkObjectForXPathInjection(item, `${path}[${index}]`));
        });
    }
    else if (typeof obj === 'object' && obj !== null) {
        for (const key in obj) {
            if (obj.hasOwnProperty(key)) {
                issues = issues.concat(checkObjectForXPathInjection(obj[key], path ? `${path}.${key}` : key));
            }
        }
    }
    return issues;
}
//# sourceMappingURL=xpathInjectionDetector.js.map