import {
  detectSQLInjection,
  isSQLSentence,
} from '../../detectors/sqlInjectionDetector';
import { detectXSS } from '../../detectors/xssDetector';
import { detectCommandInjection } from '../../detectors/commandInjectionDetector';
import { detectLDAPInjection } from '../../detectors/ldapInjectionDetector';
import { detectXPathInjection } from '../../detectors/xpathInjectionDetector';
import { detectFileInclusion } from '../../detectors/fileInclusionDetector';
import { ScanOptions } from '../../scan/interfaces/ITScanOptions';
import { ALERT_MESSAGES } from '../constants/alertConstants';

/**
 * Comprueba diferentes detectores de seguridad en un valor dado y agrega los problemas detectados a una lista de problemas.
 * Los detectores incluyen la detección de sentencias SQL, inyección de SQL, XSS (Cross-Site Scripting) e inyección de scripts.
 *
 * @param value - El valor a comprobar.
 * @param key - La clave asociada al valor.
 * @param issues - La lista de problemas detectados.
 * @param options - Las opciones de escaneo.
 * @param isKey - Indica si el valor es una clave.
 * @returns void
 *
 * @remarks
 * Este método comprueba si el valor dado contiene sentencias SQL, inyección de SQL, XSS o inyección de scripts.
 * Si se detecta alguno de estos problemas, se agrega un mensaje de alerta a la lista de problemas.
 *
 * @example
 * checkDetectors("SELECT * FROM users", "query", issues, { detectSentences: true, detectReservedWords: true }, true);
 */
export function checkDetectors(
  value: any,
  key: string,
  issues: string[],
  options: ScanOptions,
  isKey: boolean = false,
): void {
  const { detectSentences, detectReservedWords } = options;
  const keyOrValue = isKey ? 'key' : key;

  console.log(typeof value);

  if (typeof value === 'string') {
    if (detectSentences && isSQLSentence(value)) {
      issues.push(ALERT_MESSAGES.sqlSentenceDetected(keyOrValue, value));
    }
    if (detectReservedWords && detectSQLInjection(value)) {
      issues.push(ALERT_MESSAGES.sqlInjectionDetected(keyOrValue, value));
    }
    if (detectXSS(value)) {
      issues.push(ALERT_MESSAGES.xssDetected(keyOrValue, value));
    }
    if (detectCommandInjection(value)) {
      issues.push(ALERT_MESSAGES.commandInjectionDetected(keyOrValue, value));
    }
    if (detectLDAPInjection(value)) {
      issues.push(ALERT_MESSAGES.ldapInjectionDetected(keyOrValue, value));
    }
    if (detectXPathInjection(value)) {
      issues.push(ALERT_MESSAGES.xpathInjectionDetected(keyOrValue, value));
    }
    if (detectFileInclusion(value)) {
      issues.push(ALERT_MESSAGES.fileInclusionDetected(keyOrValue, value));
    }
  } else if (typeof value === 'object' && value !== null) {
    for (const nestedKey in value) {
      if (value.hasOwnProperty(nestedKey)) {
        checkDetectors(
          value[nestedKey],
          `${keyOrValue}.${nestedKey}`,
          issues,
          options,
        );
      }
    }
  } else if (Array.isArray(value)) {
    value.forEach((item, index) => {
      checkDetectors(item, `${keyOrValue}[${index}]`, issues, options, isKey);
    });
  }
}
